module GlitchHelperBlueGlitch

using ..Ahorn, Maple

@pardef BlueGlitch(x1::Integer, y1::Integer, destroyRange::Integer=16, hideDestroyRange::Bool=false, hasLines::Bool=false, hideLines::Bool=false) = Entity("GlitchHelper/BlueGlitch", x=x1, y=y1, destroyRange=destroyRange, hideDestroyRange=hideDestroyRange, hasLines=hasLines, hideLines=hideLines, nodes=Tuple{Int, Int}[])

function _finalizer(entity)
  x, y = Ahorn.position(entity)
  entity.data["nodes"] = [(x + 24, y)]
end

const placements = Ahorn.PlacementDict(
    "Blue Glitch (Glitch Helper)" => Ahorn.EntityPlacement(
        BlueGlitch,
        "point",
        Dict{String, Any}(),
        _finalizer
    )
)

Ahorn.nodeLimits(entity::BlueGlitch) = 1, 1

function Ahorn.selection(entity::BlueGlitch)
    x, y = Ahorn.position(entity)
    nodes = get(entity.data, "nodes", ())
    res = [Ahorn.Rectangle(x - 8, y - 8, 16, 16)]
    for node in nodes
        nodeX, nodeY = Int.(node)
        push!(res, Ahorn.Rectangle(nodeX - 8, nodeY - 8, 16, 16))
    end
    return res
end

function Ahorn.renderSelectedAbs(ctx::Ahorn.Cairo.CairoContext, entity::BlueGlitch, room::Maple.Room)
    startX, startY = Int(entity.data["x"]), Int(entity.data["y"])
    nodes = get(entity.data, "nodes", ())
    if length(nodes) > 0
        stopX, stopY = Int.(nodes[1])
        Ahorn.drawArrow(ctx, startX, startY, stopX, stopY, Ahorn.colors.selection_selected_fc, headLength=4)
    end
end

function Ahorn.renderAbs(ctx::Ahorn.CairoContext, entity::BlueGlitch, room::Maple.Room)
    x, y = Ahorn.position(entity)
    nx, ny = Int.(get(entity.data, "nodes", ())[1])

    destroyRange = get(entity.data, "destroyRange", 8)
    Ahorn.drawRectangle(ctx, nx-destroyRange-7, ny-destroyRange-7, 14+destroyRange*2, 14+destroyRange*2, (85,80,229,144) ./ 255, (85,80,229,200) ./ 255)

    sprite = "objects/glitch/glitchblue00"

    Ahorn.drawSprite(ctx, sprite, x, y)
end

end
