module GlitchHelperBlueGlitch

using ..Ahorn, Maple

# @mapdef Entity "GlitchHelper/BlueGlitch" BlueGlitch(x::Integer, y::Integer, destroyRange::Integer=16, hideDestroyRange::Bool=false)
@pardef PurpleGlitch(x1::Integer, y1::Integer, destroyRange::Integer=16, hideDestroyRange::Bool=false, hasLines::Bool=false, hideLines::Bool=false) = Entity("GlitchHelper/PurpleGlitch", x=x1, y=y1, destroyRange=destroyRange, hideDestroyRange=hideDestroyRange, hasLines=hasLines, hideLines=hideLines, nodes=Tuple{Int, Int}[])
# @pardef GlitchEntity(x1::Integer, y1::Integer) = Entity("GlitchHelper/Glitch", x=x1, y=y1, nodes=Tuple{Int, Int}[])


function _finalizer(entity)
  x, y = Ahorn.position(entity)
  entity.data["nodes"] = [(x + 24, y)]
end

const placements = Ahorn.PlacementDict(
    "Purple Glitch (Glitch Helper)" => Ahorn.EntityPlacement(
        PurpleGlitch,
        "point",
        Dict{String, Any}(),
        _finalizer
    )
)

sprite = "objects/glitch/glitchpurple00"

Ahorn.nodeLimits(entity::PurpleGlitch) = 1, 1

function Ahorn.selection(entity::PurpleGlitch)
    x, y = Ahorn.position(entity)
    nodes = get(entity.data, "nodes", ())
    res = [Ahorn.Rectangle(x - 8, y - 8, 16, 16)]
    for node in nodes
        nodeX, nodeY = Int.(node)
        push!(res, Ahorn.Rectangle(nodeX - 8, nodeY - 8, 16, 16))
    end
    return res
end

function Ahorn.renderSelectedAbs(ctx::Ahorn.Cairo.CairoContext, entity::PurpleGlitch, room::Maple.Room)
    startX, startY = Int(entity.data["x"]), Int(entity.data["y"])
    nodes = get(entity.data, "nodes", ())
    if length(nodes) > 0
        stopX, stopY = Int.(nodes[1])
        Ahorn.drawArrow(ctx, startX, startY, stopX, stopY, Ahorn.colors.selection_selected_fc, headLength=4)
    end
end

function Ahorn.renderAbs(ctx::Ahorn.CairoContext, entity::PurpleGlitch, room::Maple.Room)
    x, y = Ahorn.position(entity)
    nx, ny = Int.(get(entity.data, "nodes", ())[1])

    destroyRange = get(entity.data, "destroyRange", 8)
    Ahorn.drawRectangle(ctx, nx-destroyRange-7, ny-destroyRange-7, 14+destroyRange*2, 14+destroyRange*2, (148,78,228,144) ./ 255, (148,78,228,200) ./ 255)

    sprite = "objects/glitch/glitchpurple00"

    Ahorn.drawSprite(ctx, sprite, x, y)
end

end